@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -53,6 +53,7 @@
 #include <QtGui/qguiapplication.h>
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/qstylehints.h>
+#include <QtCore/qmath.h>
 #include "qplatformdefs.h"
 
 QT_BEGIN_NAMESPACE
@@ -210,7 +211,7 @@ public:
         axisData->transitionToSet = true;
 
         actions.clear();
-        actions << QQuickAction(fp->contentItem, propName, toPos);
+        actions << QQuickStateAction(fp->contentItem, propName, toPos);
         QQuickTransitionManager::transition(actions, fp->rebound, fp->contentItem);
         return true;
     }
@@ -422,12 +423,16 @@ void QQuickFlickablePrivate::fixupX_callback(void *data)
 void QQuickFlickablePrivate::fixupX()
 {
     Q_Q(QQuickFlickable);
+    if (!q->isComponentComplete())
+        return; //Do not fixup from initialization values
     fixup(hData, q->minXExtent(), q->maxXExtent());
 }
 
 void QQuickFlickablePrivate::fixupY()
 {
     Q_Q(QQuickFlickable);
+    if (!q->isComponentComplete())
+        return; //Do not fixup from initialization values
     fixup(vData, q->minYExtent(), q->maxYExtent());
 }
 
@@ -576,14 +581,14 @@ is finished.
 */
 
 /*!
-    \qmlsignal QtQuick2::Flickable::onDragStarted()
+    \qmlsignal QtQuick::Flickable::onDragStarted()
 
     This handler is called when the view starts to be dragged due to user
     interaction.
 */
 
 /*!
-    \qmlsignal QtQuick2::Flickable::onDragEnded()
+    \qmlsignal QtQuick::Flickable::onDragEnded()
 
     This handler is called when the user stops dragging the view.
 
@@ -594,7 +599,7 @@ is finished.
 /*!
     \qmltype Flickable
     \instantiates QQuickFlickable
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-input
     \ingroup qtquick-containers
 
@@ -643,14 +648,14 @@ is finished.
 */
 
 /*!
-    \qmlsignal QtQuick2::Flickable::onMovementStarted()
+    \qmlsignal QtQuick::Flickable::onMovementStarted()
 
     This handler is called when the view begins moving due to user
     interaction.
 */
 
 /*!
-    \qmlsignal QtQuick2::Flickable::onMovementEnded()
+    \qmlsignal QtQuick::Flickable::onMovementEnded()
 
     This handler is called when the view stops moving due to user
     interaction.  If a flick was generated, this handler will
@@ -660,7 +665,7 @@ is finished.
 */
 
 /*!
-    \qmlsignal QtQuick2::Flickable::onFlickStarted()
+    \qmlsignal QtQuick::Flickable::onFlickStarted()
 
     This handler is called when the view is flicked.  A flick
     starts from the point that the mouse or touch is released,
@@ -668,16 +673,16 @@ is finished.
 */
 
 /*!
-    \qmlsignal QtQuick2::Flickable::onFlickEnded()
+    \qmlsignal QtQuick::Flickable::onFlickEnded()
 
     This handler is called when the view stops moving due to a flick.
 */
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::visibleArea.xPosition
-    \qmlproperty real QtQuick2::Flickable::visibleArea.widthRatio
-    \qmlproperty real QtQuick2::Flickable::visibleArea.yPosition
-    \qmlproperty real QtQuick2::Flickable::visibleArea.heightRatio
+    \qmlproperty real QtQuick::Flickable::visibleArea.xPosition
+    \qmlproperty real QtQuick::Flickable::visibleArea.widthRatio
+    \qmlproperty real QtQuick::Flickable::visibleArea.yPosition
+    \qmlproperty real QtQuick::Flickable::visibleArea.heightRatio
 
     These properties describe the position and size of the currently viewed area.
     The size is defined as the percentage of the full view currently visible,
@@ -713,8 +718,8 @@ QQuickFlickable::~QQuickFlickable()
 }
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::contentX
-    \qmlproperty real QtQuick2::Flickable::contentY
+    \qmlproperty real QtQuick::Flickable::contentX
+    \qmlproperty real QtQuick::Flickable::contentY
 
     These properties hold the surface coordinate currently at the top-left
     corner of the Flickable. For example, if you flick an image up 100 pixels,
@@ -755,7 +760,7 @@ void QQuickFlickable::setContentY(qreal pos)
 }
 
 /*!
-    \qmlproperty bool QtQuick2::Flickable::interactive
+    \qmlproperty bool QtQuick::Flickable::interactive
 
     This property describes whether the user can interact with the Flickable.
     A user cannot drag or flick a Flickable that is not interactive.
@@ -793,8 +798,8 @@ void QQuickFlickable::setInteractive(bool interactive)
 }
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::horizontalVelocity
-    \qmlproperty real QtQuick2::Flickable::verticalVelocity
+    \qmlproperty real QtQuick::Flickable::horizontalVelocity
+    \qmlproperty real QtQuick::Flickable::verticalVelocity
 
     The instantaneous velocity of movement along the x and y axes, in pixels/sec.
 
@@ -818,10 +823,10 @@ qreal QQuickFlickable::verticalVelocity() const
 }
 
 /*!
-    \qmlproperty bool QtQuick2::Flickable::atXBeginning
-    \qmlproperty bool QtQuick2::Flickable::atXEnd
-    \qmlproperty bool QtQuick2::Flickable::atYBeginning
-    \qmlproperty bool QtQuick2::Flickable::atYEnd
+    \qmlproperty bool QtQuick::Flickable::atXBeginning
+    \qmlproperty bool QtQuick::Flickable::atXEnd
+    \qmlproperty bool QtQuick::Flickable::atYBeginning
+    \qmlproperty bool QtQuick::Flickable::atYEnd
 
     These properties are true if the flickable view is positioned at the beginning,
     or end respectively.
@@ -851,7 +856,7 @@ bool QQuickFlickable::isAtYBeginning() const
 }
 
 /*!
-    \qmlproperty Item QtQuick2::Flickable::contentItem
+    \qmlproperty Item QtQuick::Flickable::contentItem
 
     The internal item that contains the Items to be moved in the Flickable.
 
@@ -877,13 +882,15 @@ QQuickItem *QQuickFlickable::contentItem()
 QQuickFlickableVisibleArea *QQuickFlickable::visibleArea()
 {
     Q_D(QQuickFlickable);
-    if (!d->visibleArea)
+    if (!d->visibleArea) {
         d->visibleArea = new QQuickFlickableVisibleArea(this);
+        d->visibleArea->updateVisible(); // calculate initial ratios
+    }
     return d->visibleArea;
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Flickable::flickableDirection
+    \qmlproperty enumeration QtQuick::Flickable::flickableDirection
 
     This property determines which directions the view can be flicked.
 
@@ -912,6 +919,18 @@ void QQuickFlickable::setFlickableDirection(FlickableDirection direction)
     }
 }
 
+/*!
+    \qmlproperty bool QtQuick::Flickable::pixelAligned
+
+    This property sets the alignment of \l contentX and \l contentY to
+    pixels (\c true) or subpixels (\c false).
+
+    Enable pixelAligned to optimize for still content or moving content with
+    high constrast edges, such as one-pixel-wide lines, text or vector graphics.
+    Disable pixelAligned when optimizing for animation quality.
+
+    The default is \c false.
+*/
 bool QQuickFlickable::pixelAligned() const
 {
     Q_D(const QQuickFlickable);
@@ -929,16 +948,15 @@ void QQuickFlickable::setPixelAligned(bool align)
 
 qint64 QQuickFlickablePrivate::computeCurrentTime(QInputEvent *event)
 {
-    if (0 != event->timestamp() && QQuickItemPrivate::consistentTime == -1)
+    if (0 != event->timestamp())
         return event->timestamp();
-
-    return QQuickItemPrivate::elapsed(timer);
+    return timer.elapsed();
 }
 
 void QQuickFlickablePrivate::handleMousePressEvent(QMouseEvent *event)
 {
     Q_Q(QQuickFlickable);
-    QQuickItemPrivate::start(timer);
+    timer.start();
     if (interactive && timeline.isActive()
         && ((qAbs(hData.smoothVelocity.value()) > RetainGrabVelocity && !hData.fixingUp && !hData.inOvershoot)
             || (qAbs(vData.smoothVelocity.value()) > RetainGrabVelocity && !vData.fixingUp && !vData.inOvershoot))) {
@@ -964,6 +982,7 @@ void QQuickFlickablePrivate::handleMousePressEvent(QMouseEvent *event)
     q->setKeepMouseGrab(stealMouse);
     clearDelayedPress();
     pressed = true;
+
     if (hData.transitionToBounds)
         hData.transitionToBounds->stopTransition();
     if (vData.transitionToBounds)
@@ -996,8 +1015,8 @@ void QQuickFlickablePrivate::handleMousePressEvent(QMouseEvent *event)
     if (wasFlicking)
         emit q->flickingChanged();
     lastPosTime = lastPressTime = computeCurrentTime(event);
-    QQuickItemPrivate::start(vData.velocityTime);
-    QQuickItemPrivate::start(hData.velocityTime);
+    vData.velocityTime.start();
+    hData.velocityTime.start();
 }
 
 void QQuickFlickablePrivate::handleMouseMoveEvent(QMouseEvent *event)
@@ -1031,23 +1050,20 @@ void QQuickFlickablePrivate::handleMouseMoveEvent(QMouseEvent *event)
                 newY = minY + (newY - minY) / 2;
             if (newY < maxY && maxY - minY <= 0)
                 newY = maxY + (newY - maxY) / 2;
-            if (boundsBehavior == QQuickFlickable::StopAtBounds && (newY > minY || newY < maxY)) {
-                rejectY = true;
-                if (newY < maxY) {
-                    newY = maxY;
-                    rejectY = false;
-                }
-                if (newY > minY) {
-                    newY = minY;
-                    rejectY = false;
-                }
+            if (boundsBehavior == QQuickFlickable::StopAtBounds && newY <= maxY) {
+                newY = maxY;
+                rejectY = vData.pressPos == maxY && dy < 0;
+            }
+            if (boundsBehavior == QQuickFlickable::StopAtBounds && newY >= minY) {
+                newY = minY;
+                rejectY = vData.pressPos == minY && dy > 0;
             }
             if (!rejectY && stealMouse && dy != 0.0) {
                 clearTimeline();
                 vData.move.setValue(newY);
                 vMoved = true;
             }
-            if (overThreshold)
+            if (!rejectY && overThreshold)
                 stealY = true;
         }
     }
@@ -1065,24 +1081,22 @@ void QQuickFlickablePrivate::handleMouseMoveEvent(QMouseEvent *event)
                 newX = minX + (newX - minX) / 2;
             if (newX < maxX && maxX - minX <= 0)
                 newX = maxX + (newX - maxX) / 2;
-            if (boundsBehavior == QQuickFlickable::StopAtBounds && (newX > minX || newX < maxX)) {
-                rejectX = true;
-                if (newX < maxX) {
-                    newX = maxX;
-                    rejectX = false;
-                }
-                if (newX > minX) {
-                    newX = minX;
-                    rejectX = false;
-                }
+            if (boundsBehavior == QQuickFlickable::StopAtBounds && newX <= maxX) {
+                newX = maxX;
+                rejectX = hData.pressPos == maxX && dx < 0;
             }
+            if (boundsBehavior == QQuickFlickable::StopAtBounds && newX >= minX) {
+                newX = minX;
+                rejectX = hData.pressPos == minX && dx > 0;
+            }
+
             if (!rejectX && stealMouse && dx != 0.0) {
                 clearTimeline();
                 hData.move.setValue(newX);
                 hMoved = true;
             }
 
-            if (overThreshold)
+            if (!rejectX && overThreshold)
                 stealX = true;
         }
     }
@@ -1231,15 +1245,6 @@ void QQuickFlickable::mouseMoveEvent(QMouseEvent *event)
 {
     Q_D(QQuickFlickable);
     if (d->interactive) {
-        if (d->delayedPressEvent) {
-            // A move beyond the threshold replays the press to give nested Flickables
-            // the opportunity to grab the gesture.
-            QPointF delta = event->localPos() - d->delayedPressEvent->localPos();
-            if (QQuickWindowPrivate::dragOverThreshold(qAbs(delta.x()), Qt::XAxis, event)
-                || QQuickWindowPrivate::dragOverThreshold(qAbs(delta.y()), Qt::YAxis, event)) {
-                d->replayDelayedPress();
-            }
-        }
         d->handleMouseMoveEvent(event);
         event->accept();
     } else {
@@ -1460,7 +1465,7 @@ void QQuickFlickablePrivate::viewportAxisMoved(AxisData &data, qreal minExtent,
                                            QQuickTimeLineCallback::Callback fixupCallback)
 {
     if (pressed || calcVelocity) {
-        int elapsed = QQuickItemPrivate::restart(data.velocityTime);
+        int elapsed = data.velocityTime.restart();
         if (elapsed > 0) {
             qreal velocity = (data.lastPos - data.move.value()) * 1000 / elapsed;
             if (qAbs(velocity) > 0) {
@@ -1507,8 +1512,7 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
 
     bool changed = false;
     if (newGeometry.width() != oldGeometry.width()) {
-        if (xflick())
-            changed = true;
+        changed = true; // we must update visualArea.widthRatio
         if (d->hData.viewSize < 0) {
             d->contentItem->setWidth(width());
             emit contentWidthChanged();
@@ -1520,8 +1524,7 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
         }
     }
     if (newGeometry.height() != oldGeometry.height()) {
-        if (yflick())
-            changed = true;
+        changed = true; // we must update visualArea.heightRatio
         if (d->vData.viewSize < 0) {
             d->contentItem->setHeight(height());
             emit contentHeightChanged();
@@ -1538,7 +1541,7 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
 }
 
 /*!
-    \qmlmethod QtQuick2::Flickable::flick(qreal xVelocity, qreal yVelocity)
+    \qmlmethod QtQuick::Flickable::flick(qreal xVelocity, qreal yVelocity)
 
     Flicks the content with \a xVelocity horizontally and \a yVelocity vertically in pixels/sec.
 */
@@ -1573,7 +1576,7 @@ void QQuickFlickablePrivate::flickingStarted(bool flickingH, bool flickingV)
 }
 
 /*!
-    \qmlmethod QtQuick2::Flickable::cancelFlick()
+    \qmlmethod QtQuick::Flickable::cancelFlick()
 
     Cancels the current flick animation.
 */
@@ -1628,7 +1631,7 @@ QQmlListProperty<QQuickItem> QQuickFlickable::flickableChildren()
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Flickable::boundsBehavior
+    \qmlproperty enumeration QtQuick::Flickable::boundsBehavior
     This property holds whether the surface may be dragged
     beyond the Flickable's boundaries, or overshoot the
     Flickable's boundaries when flicked.
@@ -1664,7 +1667,7 @@ void QQuickFlickable::setBoundsBehavior(BoundsBehavior b)
 }
 
 /*!
-    \qmlproperty Transition QtQuick2::Flickable::rebound
+    \qmlproperty Transition QtQuick::Flickable::rebound
 
     This holds the transition to be applied to the content view when
     it snaps back to the bounds of the flickable. The transition is
@@ -1725,8 +1728,8 @@ void QQuickFlickable::setRebound(QQuickTransition *transition)
 }
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::contentWidth
-    \qmlproperty real QtQuick2::Flickable::contentHeight
+    \qmlproperty real QtQuick::Flickable::contentWidth
+    \qmlproperty real QtQuick::Flickable::contentHeight
 
     The dimensions of the content (the surface controlled by Flickable).
     This should typically be set to the combined size of the items placed in the
@@ -1807,10 +1810,10 @@ void QQuickFlickable::setContentHeight(qreal h)
 }
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::topMargin
-    \qmlproperty real QtQuick2::Flickable::leftMargin
-    \qmlproperty real QtQuick2::Flickable::bottomMargin
-    \qmlproperty real QtQuick2::Flickable::rightMargin
+    \qmlproperty real QtQuick::Flickable::topMargin
+    \qmlproperty real QtQuick::Flickable::leftMargin
+    \qmlproperty real QtQuick::Flickable::bottomMargin
+    \qmlproperty real QtQuick::Flickable::rightMargin
 
     These properties hold the margins around the content.  This space is reserved
     in addition to the contentWidth and contentHeight.
@@ -1902,8 +1905,8 @@ void QQuickFlickable::setRightMargin(qreal m)
 }
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::originX
-    \qmlproperty real QtQuick2::Flickable::originY
+    \qmlproperty real QtQuick::Flickable::originX
+    \qmlproperty real QtQuick::Flickable::originY
 
     These properties hold the origin of the content. This value always refers
     to the top-left position of the content regardless of layout direction.
@@ -1927,7 +1930,7 @@ qreal QQuickFlickable::originX() const
 
 
 /*!
-    \qmlmethod QtQuick2::Flickable::resizeContent(real width, real height, QPointF center)
+    \qmlmethod QtQuick::Flickable::resizeContent(real width, real height, QPointF center)
 
     Resizes the content to \a width x \a height about \a center.
 
@@ -1965,7 +1968,7 @@ void QQuickFlickable::resizeContent(qreal w, qreal h, QPointF center)
 }
 
 /*!
-    \qmlmethod QtQuick2::Flickable::returnToBounds()
+    \qmlmethod QtQuick::Flickable::returnToBounds()
 
     Ensures the content is within legal bounds.
 
@@ -2084,7 +2087,8 @@ bool QQuickFlickable::sendMouseEvent(QQuickItem *item, QMouseEvent *event)
         d->lastPosTime = -1;
         returnToBounds();
     }
-    if (event->type() == QEvent::MouseButtonRelease) {
+    if (event->type() == QEvent::MouseButtonRelease || (grabber && grabber->keepMouseGrab() && !grabberDisabled)) {
+        // mouse released, or another item has claimed the grab
         d->lastPosTime = -1;
         d->clearDelayedPress();
         d->stealMouse = false;
@@ -2097,7 +2101,7 @@ bool QQuickFlickable::sendMouseEvent(QQuickItem *item, QMouseEvent *event)
 bool QQuickFlickable::childMouseEventFilter(QQuickItem *i, QEvent *e)
 {
     Q_D(QQuickFlickable);
-    if (!isVisible() || !isEnabled())
+    if (!isVisible() || !isEnabled() || !isInteractive())
         return QQuickItem::childMouseEventFilter(i, e);
     switch (e->type()) {
     case QEvent::MouseButtonPress:
@@ -2118,7 +2122,7 @@ bool QQuickFlickable::childMouseEventFilter(QQuickItem *i, QEvent *e)
 }
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::maximumFlickVelocity
+    \qmlproperty real QtQuick::Flickable::maximumFlickVelocity
     This property holds the maximum velocity that the user can flick the view in pixels/second.
 
     The default value is platform dependent.
@@ -2139,7 +2143,7 @@ void QQuickFlickable::setMaximumFlickVelocity(qreal v)
 }
 
 /*!
-    \qmlproperty real QtQuick2::Flickable::flickDeceleration
+    \qmlproperty real QtQuick::Flickable::flickDeceleration
     This property holds the rate at which a flick will decelerate.
 
     The default value is platform dependent.
@@ -2166,9 +2170,9 @@ bool QQuickFlickable::isFlicking() const
 }
 
 /*!
-    \qmlproperty bool QtQuick2::Flickable::flicking
-    \qmlproperty bool QtQuick2::Flickable::flickingHorizontally
-    \qmlproperty bool QtQuick2::Flickable::flickingVertically
+    \qmlproperty bool QtQuick::Flickable::flicking
+    \qmlproperty bool QtQuick::Flickable::flickingHorizontally
+    \qmlproperty bool QtQuick::Flickable::flickingVertically
 
     These properties describe whether the view is currently moving horizontally,
     vertically or in either direction, due to the user flicking the view.
@@ -2186,9 +2190,9 @@ bool QQuickFlickable::isFlickingVertically() const
 }
 
 /*!
-    \qmlproperty bool QtQuick2::Flickable::dragging
-    \qmlproperty bool QtQuick2::Flickable::draggingHorizontally
-    \qmlproperty bool QtQuick2::Flickable::draggingVertically
+    \qmlproperty bool QtQuick::Flickable::dragging
+    \qmlproperty bool QtQuick::Flickable::draggingHorizontally
+    \qmlproperty bool QtQuick::Flickable::draggingVertically
 
     These properties describe whether the view is currently moving horizontally,
     vertically or in either direction, due to the user dragging the view.
@@ -2258,7 +2262,7 @@ bool QQuickFlickablePrivate::isViewMoving() const
 }
 
 /*!
-    \qmlproperty int QtQuick2::Flickable::pressDelay
+    \qmlproperty int QtQuick::Flickable::pressDelay
 
     This property holds the time to delay (ms) delivering a press to
     children of the Flickable.  This can be useful where reacting
@@ -2269,7 +2273,11 @@ bool QQuickFlickablePrivate::isViewMoving() const
     within the timeout, both the press and release will be delivered.
 
     Note that for nested Flickables with pressDelay set, the pressDelay of
-    outer Flickables is overridden by the innermost Flickable.
+    outer Flickables is overridden by the innermost Flickable. If the drag
+    exceeds the platform drag threshold, the press event will be delivered
+    regardless of this property.
+
+    \sa QStyleHints
 */
 int QQuickFlickable::pressDelay() const
 {
@@ -2287,9 +2295,9 @@ void QQuickFlickable::setPressDelay(int delay)
 }
 
 /*!
-    \qmlproperty bool QtQuick2::Flickable::moving
-    \qmlproperty bool QtQuick2::Flickable::movingHorizontally
-    \qmlproperty bool QtQuick2::Flickable::movingVertically
+    \qmlproperty bool QtQuick::Flickable::moving
+    \qmlproperty bool QtQuick::Flickable::movingHorizontally
+    \qmlproperty bool QtQuick::Flickable::movingVertically
 
     These properties describe whether the view is currently moving horizontally,
     vertically or in either direction, due to the user either dragging or
@@ -2368,7 +2376,7 @@ void QQuickFlickable::movementEnding(bool hMovementEnding, bool vMovementEnding)
     }
 
     // emit moving signals
-    bool wasMoving = d->hData.moving || d->vData.moving;
+    bool wasMoving = isMoving();
     if (hMovementEnding && d->hData.moving
             && (!d->pressed && !d->stealMouse)) {
         d->hData.moving = false;
@@ -2381,7 +2389,7 @@ void QQuickFlickable::movementEnding(bool hMovementEnding, bool vMovementEnding)
         d->vMoved = false;
         emit movingVerticallyChanged();
     }
-    if (wasMoving && (!d->hData.moving || !d->vData.moving)) {
+    if (wasMoving && !isMoving()) {
         emit movingChanged();
         emit movementEnded();
     }
