@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -128,7 +128,7 @@ void QQuickContents::calcGeometry(QQuickItem *changed)
 
 class QQuickTransformPrivate : public QObjectPrivate
 {
-    Q_DECLARE_PUBLIC(QQuickTransform);
+    Q_DECLARE_PUBLIC(QQuickTransform)
 public:
     static QQuickTransformPrivate* get(QQuickTransform *transform) { return transform->d_func(); }
 
@@ -197,7 +197,7 @@ public:
     void updateOpacity();
     void updateZ();
 
-signals:
+Q_SIGNALS:
     void enabledChanged(bool enabled);
     void sizeChanged(const QSize &size);
     void mipmapChanged(bool mipmap);
@@ -209,6 +209,9 @@ signals:
     void sourceRectChanged(const QRectF &sourceRect);
 
 private:
+    friend class QQuickTransformAnimatorJob;
+    friend class QQuickOpacityAnimatorJob;
+
     void activate();
     void deactivate();
     void activateEffect();
@@ -293,6 +296,8 @@ public:
     static QQuickTransform *transform_at(QQmlListProperty<QQuickTransform> *list, int);
     static void transform_clear(QQmlListProperty<QQuickTransform> *list);
 
+    void _q_resourceObjectDeleted(QObject *);
+
     enum ChangeType {
         Geometry = 0x01,
         SiblingOrder = 0x02,
@@ -363,6 +368,8 @@ public:
         Qt::MouseButtons acceptedMouseButtons;
 
         QQuickItem::TransformOrigin origin:5;
+
+        QObjectList resourcesList;
     };
     QLazilyAllocated<ExtraData> extra;
 
@@ -396,8 +403,8 @@ public:
     bool antialiasing:1;
     bool focus:1;
     bool activeFocus:1;
-    bool notifiedFocus:1;
     // Bit 16
+    bool notifiedFocus:1;
     bool notifiedActiveFocus:1;
     bool filtersChildMouseEvents:1;
     bool explicitVisible:1;
@@ -413,8 +420,8 @@ public:
     bool isAccessible:1;
     bool culled:1;
     bool hasCursor:1;
-    // bool dummy:1
     // Bit 32
+    bool activeFocusOnTab:1;
 
     enum DirtyType {
         TransformOrigin         = 0x00000001,
@@ -461,6 +468,7 @@ public:
     QQuickWindow *window;
     int windowRefCount;
     inline QSGContext *sceneGraphContext() const;
+    inline QSGRenderContext *sceneGraphRenderContext() const;
 
     QQuickItem *parentItem;
     QQmlNotifier parentNotifier;
@@ -484,6 +492,12 @@ public:
     QTransform itemToWindowTransform() const;
     void itemToParentTransform(QTransform &) const;
 
+    static bool focusNextPrev(QQuickItem *item, bool forward);
+    static QQuickItem *nextPrevItemInTabFocusChain(QQuickItem *item, bool forward);
+
+    static bool qt_tab_all_widgets(); //todo: move to QGuiApplication?
+    static bool canAcceptTabFocus(QQuickItem *item);
+
     qreal x;
     qreal y;
     qreal width;
@@ -516,7 +530,7 @@ public:
 
     void emitChildrenRectChanged(const QRectF &rect) {
         Q_Q(QQuickItem);
-        emit q->childrenRectChanged(rect);
+        Q_EMIT q->childrenRectChanged(rect);
     }
 
     QPointF computeTransformOrigin() const;
@@ -574,12 +588,6 @@ public:
 
     virtual void mirrorChange() {}
 
-    static qint64 consistentTime;
-    static void setConsistentTime(qint64 t);
-    static void start(QElapsedTimer &);
-    static qint64 elapsed(QElapsedTimer &);
-    static qint64 restart(QElapsedTimer &);
-
     void incrementCursorCount(int delta);
 };
 
@@ -749,7 +757,7 @@ public:
         Q_D(QQuickKeysAttached);
         if (enabled != d->enabled) {
             d->enabled = enabled;
-            emit enabledChanged();
+            Q_EMIT enabledChanged();
         }
     }
 
@@ -819,28 +827,9 @@ private:
     virtual void inputMethodEvent(QInputMethodEvent *, bool post);
     virtual QVariant inputMethodQuery(Qt::InputMethodQuery query) const;
 #endif
-    const QByteArray keyToSignal(int key) {
-        QByteArray keySignal;
-        if (key >= Qt::Key_0 && key <= Qt::Key_9) {
-            keySignal = "digit0Pressed";
-            keySignal[5] = '0' + (key - Qt::Key_0);
-        } else {
-            int i = 0;
-            while (sigMap[i].key && sigMap[i].key != key)
-                ++i;
-            keySignal = sigMap[i].sig;
-        }
-        return keySignal;
-    }
+    const QByteArray keyToSignal(int key);
 
     bool isConnected(const char *signalName);
-
-    struct SigMap {
-        int key;
-        const char *sig;
-    };
-
-    static const SigMap sigMap[];
 };
 
 Qt::MouseButtons QQuickItemPrivate::acceptedMouseButtons() const
@@ -850,6 +839,12 @@ Qt::MouseButtons QQuickItemPrivate::acceptedMouseButtons() const
 }
 
 QSGContext *QQuickItemPrivate::sceneGraphContext() const
+{
+    Q_ASSERT(window);
+    return static_cast<QQuickWindowPrivate *>(QObjectPrivate::get(window))->context->sceneGraphContext();
+}
+
+QSGRenderContext *QQuickItemPrivate::sceneGraphRenderContext() const
 {
     Q_ASSERT(window);
     return static_cast<QQuickWindowPrivate *>(QObjectPrivate::get(window))->context;
@@ -876,9 +871,9 @@ QSGTransformNode *QQuickItemPrivate::itemNode()
     if (!itemNodeInstance) {
         itemNodeInstance = createTransformNode();
         itemNodeInstance->setFlag(QSGNode::OwnedByParent, false);
-#ifdef QML_RUNTIME_TESTING
+#ifdef QSG_RUNTIME_DESCRIPTION
         Q_Q(QQuickItem);
-        itemNodeInstance->description = QString::fromLatin1("QQuickItem(%1)").arg(QString::fromLatin1(q->metaObject()->className()));
+        qsgnode_set_description(itemNodeInstance, QString::fromLatin1("QQuickItem(%1)").arg(QString::fromLatin1(q->metaObject()->className())));
 #endif
     }
     return itemNodeInstance;
@@ -896,14 +891,14 @@ QSGNode *QQuickItemPrivate::childContainerNode()
             opacityNode()->appendChildNode(groupNode);
         else
             itemNode()->appendChildNode(groupNode);
-#ifdef QML_RUNTIME_TESTING
-        groupNode->description = QLatin1String("group");
+#ifdef QSG_RUNTIME_DESCRIPTION
+        qsgnode_set_description(groupNode, QLatin1String("group"));
 #endif
     }
     return groupNode;
 }
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(QQuickItemPrivate::ChangeTypes);
+Q_DECLARE_OPERATORS_FOR_FLAGS(QQuickItemPrivate::ChangeTypes)
 
 QT_END_NAMESPACE
 
