@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -41,6 +41,7 @@
 
 #include "qquickitemview_p_p.h"
 #include <QtQuick/private/qquicktransition_p.h>
+#include <QtQml/QQmlInfo>
 #include "qplatformdefs.h"
 
 QT_BEGIN_NAMESPACE
@@ -50,13 +51,14 @@ QT_BEGIN_NAMESPACE
 #define QML_VIEW_DEFAULTCACHEBUFFER 320
 #endif
 
-FxViewItem::FxViewItem(QQuickItem *i, bool own, bool trackGeometry)
+FxViewItem::FxViewItem(QQuickItem *i, QQuickItemView *v, bool own)
     : item(i)
+    , view(v)
     , transitionableItem(0)
     , attached(0)
     , ownItem(own)
     , releaseAfterTransition(false)
-    , trackGeom(trackGeometry)
+    , trackGeom(false)
 {
 }
 
@@ -95,6 +97,23 @@ void FxViewItem::setVisible(bool visible)
     QQuickItemPrivate::get(item)->setCulled(!visible);
 }
 
+void FxViewItem::trackGeometry(bool track)
+{
+    if (track) {
+        if (!trackGeom) {
+            QQuickItemPrivate *itemPrivate = QQuickItemPrivate::get(item);
+            itemPrivate->addItemChangeListener(QQuickItemViewPrivate::get(view), QQuickItemPrivate::Geometry);
+            trackGeom = true;
+        }
+    } else {
+        if (trackGeom) {
+            QQuickItemPrivate *itemPrivate = QQuickItemPrivate::get(item);
+            itemPrivate->removeItemChangeListener(QQuickItemViewPrivate::get(view), QQuickItemPrivate::Geometry);
+            trackGeom = false;
+        }
+    }
+}
+
 QQuickItemViewTransitioner::TransitionType FxViewItem::scheduledTransitionType() const
 {
     return transitionableItem ? transitionableItem->nextTransitionType : QQuickItemViewTransitioner::NoTransition;
@@ -147,14 +166,14 @@ bool QQuickItemViewChangeSet::hasPendingChanges() const
     return !pendingChanges.isEmpty();
 }
 
-void QQuickItemViewChangeSet::applyChanges(const QQuickChangeSet &changeSet)
+void QQuickItemViewChangeSet::applyChanges(const QQmlChangeSet &changeSet)
 {
     pendingChanges.apply(changeSet);
 
     int moveId = -1;
     int moveOffset = 0;
 
-    foreach (const QQuickChangeSet::Remove &r, changeSet.removes()) {
+    foreach (const QQmlChangeSet::Remove &r, changeSet.removes()) {
         itemCount -= r.count;
         if (moveId == -1 && newCurrentIndex >= r.index + r.count) {
             newCurrentIndex -= r.count;
@@ -173,7 +192,7 @@ void QQuickItemViewChangeSet::applyChanges(const QQuickChangeSet &changeSet)
             currentChanged = true;
         }
     }
-    foreach (const QQuickChangeSet::Insert &i, changeSet.inserts()) {
+    foreach (const QQmlChangeSet::Insert &i, changeSet.inserts()) {
         if (moveId == -1) {
             if (itemCount && newCurrentIndex >= i.index) {
                 newCurrentIndex += i.count;
@@ -248,10 +267,7 @@ QQuickItemView::~QQuickItemView()
 QQuickItem *QQuickItemView::currentItem() const
 {
     Q_D(const QQuickItemView);
-    if (!d->currentItem)
-        return 0;
-    const_cast<QQuickItemViewPrivate*>(d)->applyPendingChanges();
-    return d->currentItem->item;
+    return d->currentItem ? d->currentItem->item : 0;
 }
 
 QVariant QQuickItemView::model() const
@@ -266,14 +282,14 @@ void QQuickItemView::setModel(const QVariant &model)
     if (d->modelVariant == model)
         return;
     if (d->model) {
-        disconnect(d->model, SIGNAL(modelUpdated(QQuickChangeSet,bool)),
-                this, SLOT(modelUpdated(QQuickChangeSet,bool)));
-        disconnect(d->model, SIGNAL(initItem(int,QQuickItem*)), this, SLOT(initItem(int,QQuickItem*)));
-        disconnect(d->model, SIGNAL(createdItem(int,QQuickItem*)), this, SLOT(createdItem(int,QQuickItem*)));
-        disconnect(d->model, SIGNAL(destroyingItem(QQuickItem*)), this, SLOT(destroyingItem(QQuickItem*)));
+        disconnect(d->model, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
+                this, SLOT(modelUpdated(QQmlChangeSet,bool)));
+        disconnect(d->model, SIGNAL(initItem(int,QObject*)), this, SLOT(initItem(int,QObject*)));
+        disconnect(d->model, SIGNAL(createdItem(int,QObject*)), this, SLOT(createdItem(int,QObject*)));
+        disconnect(d->model, SIGNAL(destroyingItem(QObject*)), this, SLOT(destroyingItem(QObject*)));
     }
 
-    QQuickVisualModel *oldModel = d->model;
+    QQmlInstanceModel *oldModel = d->model;
 
     d->clear();
     d->model = 0;
@@ -281,8 +297,8 @@ void QQuickItemView::setModel(const QVariant &model)
     d->modelVariant = model;
 
     QObject *object = qvariant_cast<QObject*>(model);
-    QQuickVisualModel *vim = 0;
-    if (object && (vim = qobject_cast<QQuickVisualModel *>(object))) {
+    QQmlInstanceModel *vim = 0;
+    if (object && (vim = qobject_cast<QQmlInstanceModel *>(object))) {
         if (d->ownModel) {
             delete oldModel;
             d->ownModel = false;
@@ -290,27 +306,27 @@ void QQuickItemView::setModel(const QVariant &model)
         d->model = vim;
     } else {
         if (!d->ownModel) {
-            d->model = new QQuickVisualDataModel(qmlContext(this), this);
+            d->model = new QQmlDelegateModel(qmlContext(this), this);
             d->ownModel = true;
             if (isComponentComplete())
-                static_cast<QQuickVisualDataModel *>(d->model.data())->componentComplete();
+                static_cast<QQmlDelegateModel *>(d->model.data())->componentComplete();
         } else {
             d->model = oldModel;
         }
-        if (QQuickVisualDataModel *dataModel = qobject_cast<QQuickVisualDataModel*>(d->model))
+        if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model))
             dataModel->setModel(model);
     }
 
     if (d->model) {
         d->bufferMode = QQuickItemViewPrivate::BufferBefore | QQuickItemViewPrivate::BufferAfter;
-        connect(d->model, SIGNAL(createdItem(int,QQuickItem*)), this, SLOT(createdItem(int,QQuickItem*)));
-        connect(d->model, SIGNAL(initItem(int,QQuickItem*)), this, SLOT(initItem(int,QQuickItem*)));
-        connect(d->model, SIGNAL(destroyingItem(QQuickItem*)), this, SLOT(destroyingItem(QQuickItem*)));
+        connect(d->model, SIGNAL(createdItem(int,QObject*)), this, SLOT(createdItem(int,QObject*)));
+        connect(d->model, SIGNAL(initItem(int,QObject*)), this, SLOT(initItem(int,QObject*)));
+        connect(d->model, SIGNAL(destroyingItem(QObject*)), this, SLOT(destroyingItem(QObject*)));
         if (isComponentComplete()) {
             d->updateSectionCriteria();
             d->refill();
             d->currentIndex = -1;
-            setCurrentIndex(0);
+            setCurrentIndex(d->model->count() > 0 ? 0 : -1);
             d->updateViewport();
 
             if (d->transitioner && d->transitioner->populateTransition) {
@@ -319,8 +335,8 @@ void QQuickItemView::setModel(const QVariant &model)
             }
         }
 
-        connect(d->model, SIGNAL(modelUpdated(QQuickChangeSet,bool)),
-                this, SLOT(modelUpdated(QQuickChangeSet,bool)));
+        connect(d->model, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
+                this, SLOT(modelUpdated(QQmlChangeSet,bool)));
         emit countChanged();
     }
     emit modelChanged();
@@ -330,7 +346,7 @@ QQmlComponent *QQuickItemView::delegate() const
 {
     Q_D(const QQuickItemView);
     if (d->model) {
-        if (QQuickVisualDataModel *dataModel = qobject_cast<QQuickVisualDataModel*>(d->model))
+        if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model))
             return dataModel->delegate();
     }
 
@@ -343,10 +359,10 @@ void QQuickItemView::setDelegate(QQmlComponent *delegate)
     if (delegate == this->delegate())
         return;
     if (!d->ownModel) {
-        d->model = new QQuickVisualDataModel(qmlContext(this));
+        d->model = new QQmlDelegateModel(qmlContext(this));
         d->ownModel = true;
     }
-    if (QQuickVisualDataModel *dataModel = qobject_cast<QQuickVisualDataModel*>(d->model)) {
+    if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model)) {
         int oldCount = dataModel->count();
         dataModel->setDelegate(delegate);
         if (isComponentComplete()) {
@@ -371,6 +387,7 @@ void QQuickItemView::setDelegate(QQmlComponent *delegate)
             emit countChanged();
     }
     emit delegateChanged();
+    d->delegateValidated = false;
 }
 
 
@@ -379,14 +396,12 @@ int QQuickItemView::count() const
     Q_D(const QQuickItemView);
     if (!d->model)
         return 0;
-    const_cast<QQuickItemViewPrivate*>(d)->applyPendingChanges();
     return d->model->count();
 }
 
 int QQuickItemView::currentIndex() const
 {
     Q_D(const QQuickItemView);
-    const_cast<QQuickItemViewPrivate*>(d)->applyPendingChanges();
     return d->currentIndex;
 }
 
@@ -496,7 +511,6 @@ QQmlComponent *QQuickItemView::header() const
 QQuickItem *QQuickItemView::headerItem() const
 {
     Q_D(const QQuickItemView);
-    const_cast<QQuickItemViewPrivate*>(d)->applyPendingChanges();
     return d->header ? d->header->item : 0;
 }
 
@@ -532,7 +546,6 @@ QQmlComponent *QQuickItemView::footer() const
 QQuickItem *QQuickItemView::footerItem() const
 {
     Q_D(const QQuickItemView);
-    const_cast<QQuickItemViewPrivate*>(d)->applyPendingChanges();
     return d->footer ? d->footer->item : 0;
 }
 
@@ -559,7 +572,6 @@ void QQuickItemView::setFooter(QQmlComponent *footerComponent)
 QQmlComponent *QQuickItemView::highlight() const
 {
     Q_D(const QQuickItemView);
-    const_cast<QQuickItemViewPrivate*>(d)->applyPendingChanges();
     return d->highlightComponent;
 }
 
@@ -579,7 +591,6 @@ void QQuickItemView::setHighlight(QQmlComponent *highlightComponent)
 QQuickItem *QQuickItemView::highlightItem() const
 {
     Q_D(const QQuickItemView);
-    const_cast<QQuickItemViewPrivate*>(d)->applyPendingChanges();
     return d->highlight ? d->highlight->item : 0;
 }
 
@@ -965,6 +976,12 @@ QQuickItem *QQuickItemView::itemAt(qreal x, qreal y) const
     return 0;
 }
 
+void QQuickItemView::forceLayout()
+{
+    Q_D(QQuickItemView);
+    d->applyPendingChanges();
+}
+
 void QQuickItemViewPrivate::applyPendingChanges()
 {
     Q_Q(QQuickItemView);
@@ -972,7 +989,7 @@ void QQuickItemViewPrivate::applyPendingChanges()
         layout();
 }
 
-int QQuickItemViewPrivate::findMoveKeyIndex(QQuickChangeSet::MoveKey key, const QVector<QQuickChangeSet::Remove> &changes) const
+int QQuickItemViewPrivate::findMoveKeyIndex(QQmlChangeSet::MoveKey key, const QVector<QQmlChangeSet::Remove> &changes) const
 {
     for (int i=0; i<changes.count(); i++) {
         for (int j=changes[i].index; j<changes[i].index + changes[i].count; j++) {
@@ -1152,7 +1169,7 @@ void QQuickItemView::destroyRemoved()
     d->forceLayoutPolish();
 }
 
-void QQuickItemView::modelUpdated(const QQuickChangeSet &changeSet, bool reset)
+void QQuickItemView::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
 {
     Q_D(QQuickItemView);
     if (reset) {
@@ -1388,7 +1405,7 @@ void QQuickItemView::componentComplete()
 {
     Q_D(QQuickItemView);
     if (d->model && d->ownModel)
-        static_cast<QQuickVisualDataModel *>(d->model.data())->componentComplete();
+        static_cast<QQmlDelegateModel *>(d->model.data())->componentComplete();
 
     QQuickFlickable::componentComplete();
 
@@ -1440,7 +1457,7 @@ QQuickItemViewPrivate::QQuickItemViewPrivate()
     , inLayout(false), inViewportMoved(false), forceLayout(false), currentIndexCleared(false)
     , haveHighlightRange(false), autoHighlight(true), highlightRangeStartValid(false), highlightRangeEndValid(false)
     , fillCacheBuffer(false), inRequest(false)
-    , runDelayedRemoveTransition(false)
+    , runDelayedRemoveTransition(false), delegateValidated(false)
 {
     bufferPause.addAnimationChangeListener(this, QAbstractAnimationJob::Completion);
     bufferPause.setLoopCount(1);
@@ -1874,8 +1891,8 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
     totalInsertionResult->visiblePos = prevViewPos;
     totalRemovalResult->visiblePos = prevViewPos;
 
-    const QVector<QQuickChangeSet::Remove> &removals = currentChanges.pendingChanges.removes();
-    const QVector<QQuickChangeSet::Insert> &insertions = currentChanges.pendingChanges.inserts();
+    const QVector<QQmlChangeSet::Remove> &removals = currentChanges.pendingChanges.removes();
+    const QVector<QQmlChangeSet::Insert> &insertions = currentChanges.pendingChanges.inserts();
     ChangeResult insertionResult(prevViewPos);
     ChangeResult removalResult(prevViewPos);
 
@@ -1895,7 +1912,7 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
         }
     }
     if (runDelayedRemoveTransition) {
-        QQuickChangeSet::Remove removal;
+        QQmlChangeSet::Remove removal;
         for (QList<FxViewItem*>::Iterator it = visibleItems.begin(); it != visibleItems.end();) {
             FxViewItem *item = *it;
             if (item->index == -1 && !item->attached->delayRemove()) {
@@ -1963,7 +1980,7 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
 
     // Whatever removed/moved items remain are no longer visible items.
     prepareRemoveTransitions(&currentChanges.removedItems);
-    for (QHash<QQuickChangeSet::MoveKey, FxViewItem *>::Iterator it = currentChanges.removedItems.begin();
+    for (QHash<QQmlChangeSet::MoveKey, FxViewItem *>::Iterator it = currentChanges.removedItems.begin();
          it != currentChanges.removedItems.end(); ++it) {
         releaseItem(it.value());
     }
@@ -1992,7 +2009,7 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
     return visibleAffected;
 }
 
-bool QQuickItemViewPrivate::applyRemovalChange(const QQuickChangeSet::Remove &removal, ChangeResult *removeResult, int *removedCount)
+bool QQuickItemViewPrivate::applyRemovalChange(const QQmlChangeSet::Remove &removal, ChangeResult *removeResult, int *removedCount)
 {
     Q_Q(QQuickItemView);
     bool visibleAffected = false;
@@ -2042,7 +2059,7 @@ bool QQuickItemViewPrivate::applyRemovalChange(const QQuickChangeSet::Remove &re
     return visibleAffected;
 }
 
-void QQuickItemViewPrivate::removeItem(FxViewItem *item, const QQuickChangeSet::Remove &removal, ChangeResult *removeResult)
+void QQuickItemViewPrivate::removeItem(FxViewItem *item, const QQmlChangeSet::Remove &removal, ChangeResult *removeResult)
 {
     if (removeResult->visiblePos.isValid()) {
         if (item->position() < removeResult->visiblePos)
@@ -2055,7 +2072,7 @@ void QQuickItemViewPrivate::removeItem(FxViewItem *item, const QQuickChangeSet::
         item->transitionNextReposition(transitioner, QQuickItemViewTransitioner::MoveTransition, true);
     } else {
         // track item so it is released later
-        currentChanges.removedItems.insertMulti(QQuickChangeSet::MoveKey(), item);
+        currentChanges.removedItems.insertMulti(QQmlChangeSet::MoveKey(), item);
     }
     if (!removeResult->changedFirstItem && item == *visibleItems.constBegin())
         removeResult->changedFirstItem = true;
@@ -2115,23 +2132,24 @@ void QQuickItemViewPrivate::prepareVisibleItemTransitions()
         return;
 
     // must call for every visible item to init or discard transitions
-    QRectF viewBounds(0, position(), q->width(), q->height());
+    QRectF viewBounds(q->contentX(), q->contentY(), q->width(), q->height());
     for (int i=0; i<visibleItems.count(); i++)
         visibleItems[i]->prepareTransition(transitioner, viewBounds);
 }
 
-void QQuickItemViewPrivate::prepareRemoveTransitions(QHash<QQuickChangeSet::MoveKey, FxViewItem *> *removedItems)
+void QQuickItemViewPrivate::prepareRemoveTransitions(QHash<QQmlChangeSet::MoveKey, FxViewItem *> *removedItems)
 {
     if (!transitioner)
         return;
 
     if (transitioner->canTransition(QQuickItemViewTransitioner::RemoveTransition, true)
             || transitioner->canTransition(QQuickItemViewTransitioner::RemoveTransition, false)) {
-        for (QHash<QQuickChangeSet::MoveKey, FxViewItem *>::Iterator it = removedItems->begin();
+        for (QHash<QQmlChangeSet::MoveKey, FxViewItem *>::Iterator it = removedItems->begin();
              it != removedItems->end(); ) {
             bool isRemove = it.key().moveId < 0;
             if (isRemove) {
                 FxViewItem *item = *it;
+                item->trackGeometry(false);
                 item->releaseAfterTransition = true;
                 releasePendingTransition.append(item);
                 item->transitionNextReposition(transitioner, QQuickItemViewTransitioner::RemoveTransition, true);
@@ -2197,7 +2215,20 @@ FxViewItem *QQuickItemViewPrivate::createItem(int modelIndex, bool asynchronous)
         requestedIndex = modelIndex;
     inRequest = true;
 
-    if (QQuickItem *item = model->item(modelIndex, asynchronous)) {
+    QObject* object = model->object(modelIndex, asynchronous);
+    QQuickItem *item = qmlobject_cast<QQuickItem*>(object);
+    if (!item) {
+        if (object) {
+            model->release(object);
+            if (!delegateValidated) {
+                delegateValidated = true;
+                QObject* delegate = q->delegate();
+                qmlInfo(delegate ? delegate : q) << QQuickItemView::tr("Delegate must be of Item type");
+            }
+        }
+        inRequest = false;
+        return 0;
+    } else {
         item->setParentItem(q->contentItem());
         if (requestedIndex == modelIndex)
             requestedIndex = -1;
@@ -2212,15 +2243,13 @@ FxViewItem *QQuickItemViewPrivate::createItem(int modelIndex, bool asynchronous)
         inRequest = false;
         return viewItem;
     }
-
-    inRequest = false;
-    return 0;
 }
 
-void QQuickItemView::createdItem(int index, QQuickItem *item)
+void QQuickItemView::createdItem(int index, QObject* object)
 {
     Q_D(QQuickItemView);
 
+    QQuickItem* item = qmlobject_cast<QQuickItem*>(object);
     if (!d->inRequest) {
         d->unrequestedItems.insert(item, index);
         d->requestedIndex = -1;
@@ -2235,17 +2264,24 @@ void QQuickItemView::createdItem(int index, QQuickItem *item)
     }
 }
 
-void QQuickItemView::initItem(int, QQuickItem *item)
+void QQuickItemView::initItem(int, QObject *object)
 {
-    item->setZ(1);
-    item->setParentItem(contentItem());
-    QQuickItemPrivate::get(item)->setCulled(true);
+    QQuickItem* item = qmlobject_cast<QQuickItem*>(object);
+    if (item) {
+        item->setZ(1);
+        item->setParentItem(contentItem());
+        QQuickItemPrivate::get(item)->setCulled(true);
+    }
 }
 
-void QQuickItemView::destroyingItem(QQuickItem *item)
+void QQuickItemView::destroyingItem(QObject *object)
 {
     Q_D(QQuickItemView);
-    d->unrequestedItems.remove(item);
+    QQuickItem* item = qmlobject_cast<QQuickItem*>(object);
+    if (item) {
+        item->setParentItem(0);
+        d->unrequestedItems.remove(item);
+    }
 }
 
 bool QQuickItemViewPrivate::releaseItem(FxViewItem *item)
@@ -2255,16 +2291,18 @@ bool QQuickItemViewPrivate::releaseItem(FxViewItem *item)
         return true;
     if (trackedItem == item)
         trackedItem = 0;
-    QQuickItemPrivate *itemPrivate = QQuickItemPrivate::get(item->item);
-    itemPrivate->removeItemChangeListener(this, QQuickItemPrivate::Geometry);
-    QQuickVisualModel::ReleaseFlags flags = model->release(item->item);
+    item->trackGeometry(false);
+
+    QQmlInstanceModel::ReleaseFlags flags = model->release(item->item);
     if (flags == 0) {
         // item was not destroyed, and we no longer reference it.
         QQuickItemPrivate::get(item->item)->setCulled(true);
         unrequestedItems.insert(item->item, model->indexOf(item->item, q));
+    } else if (flags & QQmlInstanceModel::Destroyed) {
+        item->item->setParentItem(0);
     }
     delete item;
-    return flags != QQuickVisualModel::Referenced;
+    return flags != QQmlInstanceModel::Referenced;
 }
 
 QQuickItem *QQuickItemViewPrivate::createHighlightItem()
