@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -127,8 +127,8 @@ void QQuickItemViewTransitionJob::startTransition(QQuickItemViewTransitionableIt
     }
 
     QQuickStateOperation::ActionList actions;
-    actions << QQuickAction(item->item, QLatin1String("x"), QVariant(to.x()));
-    actions << QQuickAction(item->item, QLatin1String("y"), QVariant(to.y()));
+    actions << QQuickStateAction(item->item, QLatin1String("x"), QVariant(to.x()));
+    actions << QQuickStateAction(item->item, QLatin1String("y"), QVariant(to.y()));
 
     m_transitioner->runningJobs << this;
     QQuickTransitionManager::transition(actions, trans, item->item);
@@ -568,7 +568,7 @@ QQuickViewTransitionAttached::QQuickViewTransitionAttached(QObject *parent)
 /*!
     \qmltype ViewTransition
     \instantiates QQuickViewTransitionAttached
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \brief Specifies items under transition in a view
 
@@ -856,7 +856,7 @@ QQuickViewTransitionAttached::QQuickViewTransitionAttached(QObject *parent)
   */
 
 /*!
-    \qmlattachedproperty list QtQuick2::ViewTransition::index
+    \qmlattachedproperty list QtQuick::ViewTransition::index
 
     This attached property holds the index of the item that is being
     transitioned.
@@ -866,7 +866,7 @@ QQuickViewTransitionAttached::QQuickViewTransitionAttached(QObject *parent)
 */
 
 /*!
-    \qmlattachedproperty list QtQuick2::ViewTransition::item
+    \qmlattachedproperty list QtQuick::ViewTransition::item
 
     This attached property holds the item that is being transitioned.
 
@@ -875,7 +875,7 @@ QQuickViewTransitionAttached::QQuickViewTransitionAttached(QObject *parent)
 */
 
 /*!
-    \qmlattachedproperty list QtQuick2::ViewTransition::destination
+    \qmlattachedproperty list QtQuick::ViewTransition::destination
 
     This attached property holds the final destination position for the transitioned
     item within the view.
@@ -884,7 +884,7 @@ QQuickViewTransitionAttached::QQuickViewTransitionAttached(QObject *parent)
 */
 
 /*!
-    \qmlattachedproperty list QtQuick2::ViewTransition::targetIndexes
+    \qmlattachedproperty list QtQuick::ViewTransition::targetIndexes
 
     This attached property holds a list of the indexes of the items in view
     that are the target of the relevant operation.
@@ -901,11 +901,11 @@ QQuickViewTransitionAttached::QQuickViewTransitionAttached(QObject *parent)
     \note The targetIndexes list only contains the indexes of items that are actually
     in view, or will be in the view once the relevant operation completes.
 
-    \sa QtQuick2::ViewTransition::targetIndexes
+    \sa QtQuick::ViewTransition::targetIndexes
 */
 
 /*!
-    \qmlattachedproperty list QtQuick2::ViewTransition::targetItems
+    \qmlattachedproperty list QtQuick::ViewTransition::targetItems
 
     This attached property holds the list of items in view that are the
     target of the relevant operation.
@@ -928,7 +928,7 @@ QQuickViewTransitionAttached::QQuickViewTransitionAttached(QObject *parent)
     they should not be used by ScriptAction objects in the Transition, which are
     not evaluated until the transition is run.
 
-    \sa QtQuick2::ViewTransition::targetIndexes
+    \sa QtQuick::ViewTransition::targetIndexes
 */
 QQmlListProperty<QObject> QQuickViewTransitionAttached::targetItems()
 {
