@@ -45,6 +45,7 @@
 #include <QtGui/qevent.h>
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qstylehints.h>
+#include <qpa/qplatformnativeinterface.h>
 
 #include <float.h>
 #include <math.h>
@@ -54,7 +55,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype PinchEvent
     \instantiates QQuickPinchEvent
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-input-events
     \brief For specifying information about a pinch event
 
@@ -75,9 +76,9 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty QPointF QtQuick2::PinchEvent::center
-    \qmlproperty QPointF QtQuick2::PinchEvent::startCenter
-    \qmlproperty QPointF QtQuick2::PinchEvent::previousCenter
+    \qmlproperty QPointF QtQuick::PinchEvent::center
+    \qmlproperty QPointF QtQuick::PinchEvent::startCenter
+    \qmlproperty QPointF QtQuick::PinchEvent::previousCenter
 
     These properties hold the position of the center point between the two touch points.
 
@@ -89,8 +90,8 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty real QtQuick2::PinchEvent::scale
-    \qmlproperty real QtQuick2::PinchEvent::previousScale
+    \qmlproperty real QtQuick::PinchEvent::scale
+    \qmlproperty real QtQuick::PinchEvent::previousScale
 
     These properties hold the scale factor determined by the change in distance between the two touch points.
 
@@ -103,9 +104,9 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty real QtQuick2::PinchEvent::angle
-    \qmlproperty real QtQuick2::PinchEvent::previousAngle
-    \qmlproperty real QtQuick2::PinchEvent::rotation
+    \qmlproperty real QtQuick::PinchEvent::angle
+    \qmlproperty real QtQuick::PinchEvent::previousAngle
+    \qmlproperty real QtQuick::PinchEvent::rotation
 
     These properties hold the angle between the two touch points.
 
@@ -119,10 +120,10 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty QPointF QtQuick2::PinchEvent::point1
-    \qmlproperty QPointF QtQuick2::PinchEvent::startPoint1
-    \qmlproperty QPointF QtQuick2::PinchEvent::point2
-    \qmlproperty QPointF QtQuick2::PinchEvent::startPoint2
+    \qmlproperty QPointF QtQuick::PinchEvent::point1
+    \qmlproperty QPointF QtQuick::PinchEvent::startPoint1
+    \qmlproperty QPointF QtQuick::PinchEvent::point2
+    \qmlproperty QPointF QtQuick::PinchEvent::startPoint2
 
     These properties provide the actual touch points generating the pinch.
 
@@ -133,7 +134,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty bool QtQuick2::PinchEvent::accepted
+    \qmlproperty bool QtQuick::PinchEvent::accepted
 
     Setting this property to false in the \c PinchArea::onPinchStarted handler
     will result in no further pinch events being generated, and the gesture
@@ -141,7 +142,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty int QtQuick2::PinchEvent::pointCount
+    \qmlproperty int QtQuick::PinchEvent::pointCount
 
     Holds the number of points currently touched.  The PinchArea will not react
     until two touch points have initited a gesture, but will remain active until
@@ -164,7 +165,7 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 /*!
     \qmltype PinchArea
     \instantiates QQuickPinchArea
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-input
     \inherits Item
     \brief Enables simple pinch gesture handling
@@ -189,7 +190,7 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 */
 
 /*!
-    \qmlsignal QtQuick2::PinchArea::onPinchStarted()
+    \qmlsignal QtQuick::PinchArea::onPinchStarted()
 
     This handler is called when the pinch area detects that a pinch gesture has started.
 
@@ -201,7 +202,7 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 */
 
 /*!
-    \qmlsignal QtQuick2::PinchArea::onPinchUpdated()
+    \qmlsignal QtQuick::PinchArea::onPinchUpdated()
 
     This handler is called when the pinch area detects that a pinch gesture has changed.
 
@@ -210,7 +211,7 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 */
 
 /*!
-    \qmlsignal QtQuick2::PinchArea::onPinchFinished()
+    \qmlsignal QtQuick::PinchArea::onPinchFinished()
 
     This handler is called when the pinch area detects that a pinch gesture has finished.
 
@@ -220,17 +221,17 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 
 
 /*!
-    \qmlproperty Item QtQuick2::PinchArea::pinch.target
-    \qmlproperty bool QtQuick2::PinchArea::pinch.active
-    \qmlproperty real QtQuick2::PinchArea::pinch.minimumScale
-    \qmlproperty real QtQuick2::PinchArea::pinch.maximumScale
-    \qmlproperty real QtQuick2::PinchArea::pinch.minimumRotation
-    \qmlproperty real QtQuick2::PinchArea::pinch.maximumRotation
-    \qmlproperty enumeration QtQuick2::PinchArea::pinch.dragAxis
-    \qmlproperty real QtQuick2::PinchArea::pinch.minimumX
-    \qmlproperty real QtQuick2::PinchArea::pinch.maximumX
-    \qmlproperty real QtQuick2::PinchArea::pinch.minimumY
-    \qmlproperty real QtQuick2::PinchArea::pinch.maximumY
+    \qmlproperty Item QtQuick::PinchArea::pinch.target
+    \qmlproperty bool QtQuick::PinchArea::pinch.active
+    \qmlproperty real QtQuick::PinchArea::pinch.minimumScale
+    \qmlproperty real QtQuick::PinchArea::pinch.maximumScale
+    \qmlproperty real QtQuick::PinchArea::pinch.minimumRotation
+    \qmlproperty real QtQuick::PinchArea::pinch.maximumRotation
+    \qmlproperty enumeration QtQuick::PinchArea::pinch.dragAxis
+    \qmlproperty real QtQuick::PinchArea::pinch.minimumX
+    \qmlproperty real QtQuick::PinchArea::pinch.maximumX
+    \qmlproperty real QtQuick::PinchArea::pinch.minimumY
+    \qmlproperty real QtQuick::PinchArea::pinch.maximumY
 
     \c pinch provides a convenient way to make an item react to pinch gestures.
 
@@ -246,18 +247,22 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 
 QQuickPinchArea::QQuickPinchArea(QQuickItem *parent)
   : QQuickItem(*(new QQuickPinchAreaPrivate), parent)
+  , _currentWindow(0)
 {
     Q_D(QQuickPinchArea);
     d->init();
     setAcceptedMouseButtons(Qt::LeftButton);
     setFiltersChildMouseEvents(true);
+#ifdef Q_OS_MAC
+    connect(this, &QQuickItem::windowChanged, this, &QQuickPinchArea::setTouchEventsEnabledForWindow);
+#endif
 }
 
 QQuickPinchArea::~QQuickPinchArea()
 {
 }
 /*!
-    \qmlproperty bool QtQuick2::PinchArea::enabled
+    \qmlproperty bool QtQuick::PinchArea::enabled
     This property holds whether the item accepts pinch gestures.
 
     This property defaults to true.
@@ -539,6 +544,29 @@ QQuickPinch *QQuickPinchArea::pinch()
     return d->pinch;
 }
 
+void QQuickPinchArea::setTouchEventsEnabledForWindow(QWindow *window)
+{
+#ifdef Q_OS_MAC
+    // Resolve function for enabling touch events from the (cocoa) platform plugin.
+    typedef void (*RegisterTouchWindowFunction)(QWindow *, bool);
+    RegisterTouchWindowFunction registerTouchWindow = reinterpret_cast<RegisterTouchWindowFunction>(
+        QGuiApplication::platformNativeInterface()->nativeResourceFunctionForIntegration("registertouchwindow"));
+    if (!registerTouchWindow)
+        return; // Not necessarily an error, Qt migh be using a different platform plugin.
+
+    // Disable touch on the old window, enable on the new window.
+    if (_currentWindow)
+        registerTouchWindow(_currentWindow, false);
+    if (window)
+        registerTouchWindow(window, true);
+    // Save the current window, setTouchEventsEnabledForWindow will be called
+    // with a null window on disable.
+    _currentWindow = window;
+#else // Q_OS_MAC
+    Q_UNUSED(window)
+#endif
+}
+
 
 QT_END_NAMESPACE
 
