@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -46,6 +46,7 @@
 #include "qsggeometry_p.h"
 
 #include <private/qsgadaptationlayer_p.h>
+#include <private/qsgshadersourcebuilder_p.h>
 
 #include <QOpenGLShaderProgram>
 #include <qopenglframebufferobject.h>
@@ -53,6 +54,8 @@
 
 #include <qdatetime.h>
 
+#include <private/qqmlprofilerservice_p.h>
+
 QT_BEGIN_NAMESPACE
 
 //#define RENDERER_DEBUG
@@ -60,12 +63,11 @@ QT_BEGIN_NAMESPACE
 
 
 
-#define QSG_RENDERER_TIMING
-#ifdef QSG_RENDERER_TIMING
-static bool qsg_render_timing = !qgetenv("QML_RENDERER_TIMING").isEmpty();
-static QTime frameTimer;
-static int preprocessTime;
-static int updatePassTime;
+#ifndef QSG_NO_RENDER_TIMING
+static bool qsg_render_timing = !qgetenv("QSG_RENDER_TIMING").isEmpty();
+static QElapsedTimer frameTimer;
+static qint64 preprocessTime;
+static qint64 updatePassTime;
 #endif
 
 void QSGBindable::clear(QSGRenderer::ClearMode mode) const
@@ -132,12 +134,13 @@ void QSGBindableFboId::bind() const
  */
 
 
-QSGRenderer::QSGRenderer(QSGContext *context)
+QSGRenderer::QSGRenderer(QSGRenderContext *context)
     : QObject()
     , m_clear_color(Qt::transparent)
     , m_clear_mode(ClearColorBuffer | ClearDepthBuffer)
     , m_current_opacity(1)
     , m_current_determinant(1)
+    , m_device_pixel_ratio(1)
     , m_current_stencil_value(0)
     , m_context(context)
     , m_root_node(0)
@@ -159,20 +162,6 @@ QSGRenderer::~QSGRenderer()
     delete m_node_updater;
 }
 
-/*!
-    Returns the scene graph context for this renderer.
-
-    \internal
- */
-
-QSGContext *QSGRenderer::context()
-{
-    return m_context;
-}
-
-
-
-
 /*!
     Returns the node updater that this renderer uses to update states in the
     scene graph.
@@ -237,20 +226,21 @@ void QSGRenderer::renderScene(const QSGBindable &bindable)
     m_is_rendering = true;
 
 
-#ifdef QSG_RENDERER_TIMING
-    if (qsg_render_timing)
+#ifndef QSG_NO_RENDER_TIMING
+    bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+    if (profileFrames)
         frameTimer.start();
-    int bindTime = 0;
-    int renderTime = 0;
+    qint64 bindTime = 0;
+    qint64 renderTime = 0;
 #endif
 
     m_bindable = &bindable;
     preprocess();
 
     bindable.bind();
-#ifdef QSG_RENDERER_TIMING
-    if (qsg_render_timing)
-        bindTime = frameTimer.elapsed();
+#ifndef QSG_NO_RENDER_TIMING
+    if (profileFrames)
+        bindTime = frameTimer.nsecsElapsed();
 #endif
 
 #ifndef QT_NO_DEBUG
@@ -269,9 +259,9 @@ void QSGRenderer::renderScene(const QSGBindable &bindable)
 #endif
 
     render();
-#ifdef QSG_RENDERER_TIMING
-    if (qsg_render_timing)
-        renderTime = frameTimer.elapsed();
+#ifndef QSG_NO_RENDER_TIMING
+    if (profileFrames)
+        renderTime = frameTimer.nsecsElapsed();
 #endif
 
     glDisable(GL_SCISSOR_TEST);
@@ -289,15 +279,25 @@ void QSGRenderer::renderScene(const QSGBindable &bindable)
         m_index_buffer_bound = false;
     }
 
-#ifdef QSG_RENDERER_TIMING
+#ifndef QSG_NO_RENDER_TIMING
     if (qsg_render_timing) {
-        printf(" - Breakdown of frametime: preprocess=%d, updates=%d, binding=%d, render=%d, total=%d\n",
-               preprocessTime,
-               updatePassTime - preprocessTime,
-               bindTime - updatePassTime,
-               renderTime - bindTime,
-               renderTime);
+        printf(" - Breakdown of render time: preprocess=%d, updates=%d, binding=%d, render=%d, total=%d\n",
+               int(preprocessTime / 1000000),
+               int((updatePassTime - preprocessTime) / 1000000),
+               int((bindTime - updatePassTime) / 1000000),
+               int((renderTime - bindTime) / 1000000),
+               int(renderTime / 1000000));
     }
+
+    if (QQmlProfilerService::enabled) {
+        QQmlProfilerService::sceneGraphFrame(
+                    QQmlProfilerService::SceneGraphRendererFrame,
+                    preprocessTime,
+                    updatePassTime - preprocessTime,
+                    bindTime - updatePassTime,
+                    renderTime - bindTime);
+    }
+
 #endif
 }
 
@@ -379,17 +379,17 @@ void QSGRenderer::preprocess()
         }
     }
 
-#ifdef QSG_RENDERER_TIMING
-    if (qsg_render_timing)
-        preprocessTime = frameTimer.elapsed();
+#ifndef QSG_NO_RENDER_TIMING
+    bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+    if (profileFrames)
+        preprocessTime = frameTimer.nsecsElapsed();
 #endif
 
-    nodeUpdater()->setToplevelOpacity(context()->renderAlpha());
     nodeUpdater()->updateStates(m_root_node);
 
-#ifdef QSG_RENDERER_TIMING
-    if (qsg_render_timing)
-        updatePassTime = frameTimer.elapsed();
+#ifndef QSG_NO_RENDER_TIMING
+    if (profileFrames)
+        updatePassTime = frameTimer.nsecsElapsed();
 #endif
 
 }
@@ -482,16 +482,10 @@ QSGRenderer::ClipType QSGRenderer::updateStencilClip(const QSGClipNode *clip)
         } else {
             if (!(clipType & StencilClip)) {
                 if (!m_clip_program.isLinked()) {
-                    m_clip_program.addShaderFromSourceCode(QOpenGLShader::Vertex,
-                        "attribute highp vec4 vCoord;       \n"
-                        "uniform highp mat4 matrix;         \n"
-                        "void main() {                      \n"
-                        "    gl_Position = matrix * vCoord; \n"
-                        "}");
-                    m_clip_program.addShaderFromSourceCode(QOpenGLShader::Fragment,
-                        "void main() {                                   \n"
-                        "    gl_FragColor = vec4(0.81, 0.83, 0.12, 1.0); \n" // Trolltech green ftw!
-                        "}");
+                    QSGShaderSourceBuilder::initializeProgramFromFiles(
+                        &m_clip_program,
+                        QStringLiteral(":/scenegraph/shaders/stencilclip.vert"),
+                        QStringLiteral(":/scenegraph/shaders/stencilclip.frag"));
                     m_clip_program.bindAttributeLocation("vCoord", 0);
                     m_clip_program.link();
                     m_clip_matrix_id = m_clip_program.uniformLocation("matrix");
@@ -748,7 +742,7 @@ void QSGNodeDumper::dump(QSGNode *n)
 
 void QSGNodeDumper::visitNode(QSGNode *n)
 {
-    qDebug() << QString(m_indent * 2, QLatin1Char(' ')) << n;
+    qDebug() << QByteArray(m_indent * 2, ' ').constData() << n;
     QSGNodeVisitor::visitNode(n);
 }
 
